"""
Script for building boundary to run tsunami inundation scenario for busselton,
WA, Australia.  The boundary is based on the National Hazard Map.

Input: order_filename from project.py
       event_number needs to be reflected in the project file 
Output: creates a sts file and csv files stored in project.boundaries_dir_event.
The run_busselton.py is reliant on the output of this script.
"""
import os
from os import sep 


from anuga.utilities.numerical_tools import ensure_numeric
from Scientific.IO.NetCDF import NetCDFFile
from Numeric import asarray,transpose,sqrt,argmax,argmin,arange,Float,\
    compress,zeros,fabs,allclose,ones
from anuga.utilities.polygon import inside_polygon,read_polygon
from time import localtime, strftime, gmtime
from anuga.shallow_water.data_manager import urs2sts,create_sts_boundary

try:
    from pylab import plot,show,xlabel,ylabel,legend,title,savefig,hold
except:
    print 'Cannot import pylab plotting will not work. Csv files are still created'

#no project file
#import project
from os import getenv
muxhome= getenv('MUXHOME')
home = getenv('INUNDATIONHOME') + sep +'data'+sep
state = 'western_australia'
scenario_name = 'dampier'
scenario = 'dampier_tsunami_scenario_2008'
anuga_dir= home+state+sep+scenario+sep+'anuga'+sep
boundaries_dir = anuga_dir+'boundaries'+ sep 
order_filename_dir = boundaries_dir + 'thinned_boundary_ordering.csv'
boundaries_dir_event = boundaries_dir + '27283' + sep
tide = 0
#--------------------------------------------------------------------------
# Create sts boundary from mux2files 
#--------------------------------------------------------------------------

dir=os.path.join(muxhome,'mux')

# Refer to event_027255.list in home+state+sep+event08 for event details
# taken from David's event list for 27255
urs_filenames = [
    os.path.join(dir,'Java-0000-z.grd'),
    os.path.join(dir,'Java-0001-z.grd'),
    os.path.join(dir,'Java-0002-z.grd'),
    os.path.join(dir,'Java-0003-z.grd'),
    os.path.join(dir,'Java-0005-z.grd'), 
    os.path.join(dir,'Java-0006-z.grd'), 
    os.path.join(dir,'Java-0007-z.grd'), 
    os.path.join(dir,'Java-0008-z.grd'), 
    os.path.join(dir,'Java-0010-z.grd'), 
    os.path.join(dir,'Java-0011-z.grd'), 
    os.path.join(dir,'Java-0012-z.grd'), 
    os.path.join(dir,'Java-0013-z.grd'),  
    os.path.join(dir,'Java-0015-z.grd'),  
    os.path.join(dir,'Java-0016-z.grd'),  
    os.path.join(dir,'Java-0017-z.grd'),  
    os.path.join(dir,'Java-0018-z.grd'),  
    os.path.join(dir,'Java-0020-z.grd'),  
    os.path.join(dir,'Java-0021-z.grd'),  
    os.path.join(dir,'Java-0022-z.grd'),  
    os.path.join(dir,'Java-0023-z.grd'),  
    os.path.join(dir,'Java-0025-z.grd'),  
    os.path.join(dir,'Java-0026-z.grd'),  
    os.path.join(dir,'Java-0027-z.grd'),  
    os.path.join(dir,'Java-0028-z.grd'),  
    os.path.join(dir,'Java-0029-z.grd'),  
    os.path.join(dir,'Java-0030-z.grd'),  
    os.path.join(dir,'Java-0031-z.grd'),  
    os.path.join(dir,'Java-0032-z.grd'),  
    os.path.join(dir,'Java-0033-z.grd'),  
    os.path.join(dir,'Java-0034-z.grd'),  
    os.path.join(dir,'Java-0035-z.grd'),  
    os.path.join(dir,'Java-0036-z.grd'),  
    os.path.join(dir,'Java-0037-z.grd'),  
    os.path.join(dir,'Java-0038-z.grd'),  
    os.path.join(dir,'Java-0039-z.grd'),  
    os.path.join(dir,'Java-0040-z.grd'),  
    os.path.join(dir,'Java-0041-z.grd'),  
    os.path.join(dir,'Java-0042-z.grd'),  
    os.path.join(dir,'Java-0043-z.grd'),  
    os.path.join(dir,'Java-0044-z.grd'),  
    os.path.join(dir,'Java-0045-z.grd'),  
    os.path.join(dir,'Java-0046-z.grd'),  
    os.path.join(dir,'Java-0047-z.grd'),
    os.path.join(dir,'Java-0048-z.grd')]    
 
print 'number of sources', len(urs_filenames)

if len(urs_filenames) == 44: #change per event

# AS per David Burbidge email on Friday 4th July the mag 9.3 event
# has 1m worth of slip on each sub fault therefore mutliple each unit
# source by the slip (10.4544) and sum the 44 time series together to
# get the time series for this event at the points on your boundary. 

    weight_factor = 10.4544 #change per event
    weights=weight_factor*ones(len(urs_filenames),Float)

    #scenario_name=project.scenario_name  
    order_filename=os.path.join(order_filename_dir) #project.
    #boundaries_dir_event =project.boundaries_dir_event

    print 'reading', order_filename
    # Create ordered sts file
    print 'creating sts file'

    urs2sts(urs_filenames,
            basename_out=os.path.join(boundaries_dir_event,scenario_name), 
            ordering_filename=order_filename,
            weights=weights,
            mean_stage=tide,
            verbose=True)
else:
    print 'number of sources do not match event.list'

#------------------------------------------------------------------------------
# Get gauges (timeseries of index points)
#------------------------------------------------------------------------------
def get_sts_gauge_data(filename,verbose=False):
    from Numeric import asarray,transpose,sqrt,argmax,argmin,arange,Float,\
        compress,zeros,fabs,take
    fid = NetCDFFile(filename+'.sts', 'r')    #Open existing file for read
    permutation = fid.variables['permutation'][:]
    x = fid.variables['x'][:]+fid.xllcorner   #x-coordinates of vertices
    y = fid.variables['y'][:]+fid.yllcorner   #y-coordinates of vertices
    points=transpose(asarray([x.tolist(),y.tolist()]))
    time=fid.variables['time'][:]+fid.starttime
    elevation=fid.variables['elevation'][:]
       
    basename='sts_gauge'
    quantity_names=['stage','xmomentum','ymomentum']
    quantities = {}
    for i, name in enumerate(quantity_names):
        quantities[name] = fid.variables[name][:]

#------------------------------------------------------------------------------
# Get Maxium wave height throughout timeseries at each index point
#------------------------------------------------------------------------------

    maxname = 'max_sts_stage.csv'
    fid_max = open(boundaries_dir_event+sep+maxname,'w') 
    s = 'index, x, y, max_stage \n'
    fid_max.write(s)    
    for j in range(len(x)):
        index= permutation[j]
        stage = quantities['stage'][:,j]
        xmomentum = quantities['xmomentum'][:,j]
        ymomentum = quantities['ymomentum'][:,j]

        s = '%d, %.6f, %.6f, %.6f\n' %(index, x[j], y[j], max(stage))
        fid_max.write(s)
      
#------------------------------------------------------------------------------
# Get Minium wave height throughout timeseries at each index point
#------------------------------------------------------------------------------

    minname = 'min_sts_stage.csv'
    fid_min = open(boundaries_dir_event+sep+minname,'w')
    s = 'index, x, y, max_stage \n'
    fid_min.write(s)    
    for j in range(len(x)):
        index= permutation[j]
        stage = quantities['stage'][:,j]
        xmomentum = quantities['xmomentum'][:,j]
        ymomentum = quantities['ymomentum'][:,j]

        s = '%d, %.6f, %.6f, %.6f\n' %(index, x[j], y[j], min(stage))
        fid_min.write(s)
        


        fid_sts = open(boundaries_dir_event+sep+basename+'_'+ str(index)+'.csv', 'w')
        s = 'time, stage, xmomentum, ymomentum \n'
        fid_sts.write(s)

#------------------------------------------------------------------------------
# End of the get gauges
#------------------------------------------------------------------------------
        for k in range(len(time)-1):
            s = '%.6f, %.6f, %.6f, %.6f\n' %(time[k], stage[k], xmomentum[k], ymomentum[k])
            fid_sts.write(s)

        fid_sts.close()      

    fid.close()


    return quantities,elevation,time

quantities,elevation,time=get_sts_gauge_data(os.path.join(boundaries_dir_event,scenario_name),verbose=False)

print len(elevation), len(quantities['stage'][0,:])

